import os
import argparse

from netCDF4 import Dataset

from apres import ApRESFile

def main():
    parser = argparse.ArgumentParser(description='recover the raw ApRES file from a converted netCDF4 file')
    parser.add_argument('infile', help='converted netCDF file of raw ApRES data')
    parser.add_argument('outfile', help='ApRES raw file', default=None, nargs='?')
    args = parser.parse_args()

    # We can optionally be given an explicit output filename 
    if not args.outfile:
        args.outfile = os.path.splitext(args.infile)[0] + ApRESFile.DEFAULTS['apres_suffix']

    fout = ApRESFile()

    with Dataset(args.infile, 'r') as fin:
        # Convert each netCDF group to an ApRESBurst object
        for key in fin.groups:
            burst = fout.nc_object_to_burst(fin.groups[key])
            fout.bursts.append(burst)

    fout.to_apres_dat(args.outfile)

if __name__ == '__main__':
    main()




###############################################################################
# Project: ApRES
# Purpose: Classes to encapsulate an ApRES file
# Author:  Paul M. Breen
# Date:    2018-09-24
###############################################################################

__version__ = '0.1.0'

import re
import os
import sys
import warnings

import numpy as np
from netCDF4 import Dataset

class ApRESBurst(object):
    """
    Class for reading and writing ApRES bursts
    """

    DEFAULTS = {
        'autodetect_file_format_version': True,
        'forgive': True,
        'header_markers': {
            'start': '\r\n*** Burst Header ***',
            'end': '\r\n*** End Header ***'
        },
        'end_of_header_re': '\*\*\* End Header',
        'header_line_delim': '=',
        'header_line_eol': '\r\n',
        'data_type_key': 'Average',
        'data_types': ['<u2','<u2','<u4'],
        'data_dim_keys': ['NSubBursts', 'N_ADC_SAMPLES'],
        'data_dim_order': 'C'
    }

    ALTERNATIVES = [{
            'header_line_delim': '=',
            'data_dim_keys': ['NSubBursts', 'N_ADC_SAMPLES']
        }, {
            'header_line_delim': ':',
            'data_dim_keys': ['SubBursts in burst', 'Samples']
        }
    ]

    def __init__(self, fp=None):
        """
        Constructor

        * fp is not None: Then header_start will be set to the file's current
        position.  This allows reading through a file one burst block at a
        time.  This is the normal behaviour.
        * fp is None: Allows the burst object to be populated by other means,
        rather than reading from a file

        :param fp: The open file object of the input file
        :type fp: file object
        """

        self.header_start = 0
        self.data_start = -1
        self.header_lines = []
        self.header = {}
        self.data_shape = ()
        self.data_type = '<u2'
        self.data = None

        self.fp = fp

        if self.fp:
            self.header_start = self.fp.tell()

    def reset_init_defaults(self):
        """
        Reset the initial DEFAULTS parsing tokens from the ALTERNATIVES list
        """

        for key in self.ALTERNATIVES[0]:
            self.DEFAULTS[key] = self.ALTERNATIVES[0][key]

    def read_header_lines(self):
        """
        Read the raw header lines from the file

        :returns: The raw header lines
        :rtype: list
        """

        self.data_start = -1
        self.header_lines = []

        self.fp.seek(self.header_start, 0)
        line = self.fp.readline()
        self.header_lines.append(line.rstrip())

        while(line):
            # The data section follows the end of header marker
            if re.match(self.DEFAULTS['end_of_header_re'], line):
                self.data_start = self.fp.tell()
                break

            line = self.fp.readline()
            self.header_lines.append(line.rstrip())

        return self.header_lines

    def determine_file_format_version(self):
        """
        Determine the file format version from the read header lines

        The header lines are inspected to determine the file format version,
        and the DEFAULTS parsing tokens are setup accordingly
        """

        # Create a list of the first data dimension key from the alternative
        # file formats
        dim_keys = [item['data_dim_keys'][0] for item in self.ALTERNATIVES]

        # Search the header lines for one of the keys, to determine version
        for line in self.header_lines:
            for i, key in enumerate(dim_keys):
                if re.match(key, line):
                    for key in self.ALTERNATIVES[i]:
                        self.DEFAULTS[key] = self.ALTERNATIVES[i][key]
                    return

    def store_header(self):
        """
        Store the read header lines as parsed key/value pairs

        :returns: The parsed header
        :rtype: dict
        """

        self.header = {}

        for line in self.header_lines:
            item = line.split(self.DEFAULTS['header_line_delim'], maxsplit=1)

            if len(item) > 1 and item[0]:
                self.header[item[0].strip()] = item[1].strip()

        return self.header

    def define_data_shape(self):
        """
        Parse the data dimensions from the header to define the data shape

        If the Average header item is non-zero, then that indicates that the
        subbursts have been aggregated to a single value, so we rewrite that
        dimension as 1

        :returns: The data shape
        :rtype: tuple
        """

        self.data_shape = ()
        data_shape = []

        for key in self.DEFAULTS['data_dim_keys']:
            data_shape.append(int(self.header[key]))

        if int(self.header[self.DEFAULTS['data_type_key']]) > 0:
            data_shape[0] = 1

        self.data_shape = tuple(data_shape)

        return self.data_shape

    def define_data_type(self):
        """
        The Average item from the header is used to define the data type

        Average = 0: Record contains all subbursts as 16-bit ints
        Average = 1: Record contains the averaged subbursts as a 16-bit int
        Average = 2: Record contains the stacked subbursts as a 32-bit int

        :returns: The data type
        :rtype: str
        """

        try:
            self.data_type = self.DEFAULTS['data_types'][int(self.header[self.DEFAULTS['data_type_key']])]
        except IndexError as e:
            raise IndexError('Unsupported Averaging/Stacking configuration option found in header: {}'.format(int(self.header[self.DEFAULTS['data_type_key']])))

        return self.data_type

    def configure_from_header(self):
        """
        Configure the object from the raw header lines

        :returns: The parsed header
        :rtype: dict
        """

        if self.DEFAULTS['autodetect_file_format_version']:
            self.determine_file_format_version()

        self.store_header()
        self.define_data_shape()
        self.define_data_type()

        return self.header

    def read_header(self):
        """
        Read the header from the file

        * The raw header lines are available in the header_lines list
        * The parsed header is available in the header dict
        * The file offset to the start of the data is available in data_start
        * The data shape is available in the data_shape tuple
        * The data type is available in data_type

        :returns: The parsed header
        :rtype: dict
        """

        # The header lines are used to configure this object
        self.read_header_lines()
        self.configure_from_header()

        return self.header

    def reshape_data(self):
        """
        Reshape the data according to the data_shape tuple

        If the data read from the file don't conform to the expected
        data_shape as determined from the header, then when reshaping the data,
        an exception will occur:

        * If the reshaping fails because the data array is shorter than
        expected, then we will reraise the exception.
        * If forgive mode is True, then we will emit a warning and then
        truncate the data to conform to the data_shape, otherwise we will
        reraise the exception.

        :returns: The data
        :rtype: numpy.array
        """

        try:
            self.data = np.reshape(self.data, self.data_shape, order=self.DEFAULTS['data_dim_order'])
        except ValueError as e:
            expected_len = self.data_shape[0] * self.data_shape[1]

            if self.data.size < expected_len:
                warnings.warn("Data array read from file doesn't match data_shape as read from the file header: {}. It is shorter than expected. Cannot continue.")
                raise
            elif self.data.size > expected_len:
                warnings.warn("Data array read from file doesn't match data_shape as read from the file header. It is longer than expected.")

                if self.DEFAULTS['forgive']:
                    warnings.warn("{}. Will truncate data to fit to data_shape.".format(e))
                    self.data = np.reshape(self.data[:expected_len], self.data_shape, order=self.DEFAULTS['data_dim_order'])
                else:
                    raise

        return self.data

    def read_data(self):
        """
        Read the data from the file

        The data are available in the data array, shaped according to the
        data_shape tuple

        :returns: The data
        :rtype: numpy.array
        """

        if self.data_start == -1:
            self.read_header()

        count = self.data_shape[0] * self.data_shape[1]
        self.fp.seek(self.data_start, 0)
        self.data = np.fromfile(self.fp, dtype=np.dtype(self.data_type), count=count)
        self.reshape_data()

        return self.data

    def format_header_line(self, key, value):
        """
        Format a raw header line from the given key and value

        :param key: The header item key
        :type key: str
        :param value: The header item value
        :type value: str
        :returns: The formatted raw header line
        :rtype: str
        """

        return '{}{}{}'.format(key, self.DEFAULTS['header_line_delim'], value)

    def reconstruct_header_lines(self):
        """
        Reconstruct the raw header lines from the parsed header

        :returns: The reconstructed raw header lines
        :rtype: list
        """

        self.header_lines = [self.format_header_line(k,v) for k,v in self.header.items()]
        self.header_lines.insert(0, self.DEFAULTS['header_markers']['start'])
        self.header_lines.append(self.DEFAULTS['header_markers']['end'])

        return self.header_lines

    def write_header(self, fp, subbursts=None, samples=None):
        """
        Write the header to the given file object

        :param fp: The open file object of the output file
        :type fp: file object
        :param subbursts: A range specifying the subbursts to be written
        :type subbursts: range object
        :param samples: A range specifying the samples to be written
        :type samples: range object
        """

        if self.data_start == -1:
            self.read_header()

        eol = self.DEFAULTS['header_line_eol']

        for line in self.header_lines:
            # Ensure requested subbursts & samples are reflected in the header
            if subbursts and re.match(self.DEFAULTS['data_dim_keys'][0], line):
                line = self.format_header_line(self.DEFAULTS['data_dim_keys'][0], len(subbursts))
            if samples and re.match(self.DEFAULTS['data_dim_keys'][1], line):
                line = self.format_header_line(self.DEFAULTS['data_dim_keys'][1], len(samples))

            fp.write(line + eol)

    def write_data(self, fp, subbursts=None, samples=None):
        """
        Write the data to the given file object

        :param fp: The open file object of the output file
        :type fp: file object
        :param subbursts: A range specifying the subbursts to be written
        :type subbursts: range object
        :param samples: A range specifying the samples to be written
        :type samples: range object
        """

        if self.data_start == -1:
            self.read_data()

        if not subbursts:
            subbursts = range(self.data_shape[0])
        if not samples:
            samples = range(self.data_shape[1])

        fp.write(np.asarray(self.data[subbursts.start:subbursts.stop:subbursts.step, samples.start:samples.stop:samples.step], order=self.DEFAULTS['data_dim_order']))

class ApRESFile(object):
    """
    Context manager for reading and writing ApRES files
    """

    DEFAULTS = {
        'file_encoding': 'latin-1',
        'apres_suffix': '.dat',
        'netcdf_suffix': '.nc',
        'netcdf_add_history': True,
        'netcdf_group_name': 'burst{:d}',
        'netcdf_var_name': 'data',
        'netcdf_attrs': {
            'units': '1',
            'long_name': 'Sub Burst ADC Samples'
        }
    }

    def __init__(self, path=None):
        """
        Constructor

        :param path: Path to the file
        :type path: str
        """

        self.path = path
        self.fp = None
        self.file_size = -1
        self.bursts = []

    def __enter__(self):
        """
        Enter the runtime context for this object

        The file is opened

        :returns: This object
        :rtype: ApRESFile
        """

        return self.open(self.path)

    def __exit__(self, exc_type, exc_value, exc_traceback):
        """
        Exit the runtime context for this object

        The file is closed

        :returns: False
        :rtype: bool
        """

        self.close()

        return False         # This ensures any exception is re-raised

    def open(self, path=None, mode='r'):
        """
        Open the given file

        :param path: Path to the file
        :type path: str
        :param mode: Mode in which to open the file
        :type mode: str
        :returns: This object
        :rtype: ApRESFile
        """

        if path:
            self.path = path

        self.fp = open(self.path, mode, encoding=self.DEFAULTS['file_encoding'])
        self.file_size = os.fstat(self.fp.fileno()).st_size

        return self

    def close(self):
        """
        Close the file

        :returns: This object
        :rtype: ApRESFile
        """

        self.fp.close()
        self.fp = None

        return self

    def eof(self):
        """
        Report whether the end-of-file has been reached

        :returns: True if EOF has been reached, False otherwise
        :rtype: bool
        """

        return self.fp.tell() >= self.file_size

    def read_burst(self):
        """
        Read a burst block from the file

        :returns: The burst
        :rtype: ApRESBurst
        """

        burst = ApRESBurst(fp=self.fp)
        burst.read_data()

        return burst

    def read(self):
        """
        Reads all burst blocks from the file

        The bursts are available in the bursts list

        :returns: The bursts
        :rtype: list
        """

        while not self.eof():
            burst = self.read_burst()
            self.bursts.append(burst)

        return self.bursts

    def to_apres_dat(self, path, bursts=None, subbursts=None, samples=None):
        """
        Write the bursts to the given file path as an ApRES .dat file

        When rewriting the ApRES data to a new file, the bursts, subbursts,
        and the ADC samples for those selected subbursts, can be subsetted.
        The bursts, subbursts, and samples keyword arguments must specify a
        range object

        :param path: The path of the output file
        :type path: str
        :param bursts: A range specifying the bursts to be written
        :type bursts: range object
        :param subbursts: A range specifying the subbursts to be written
        :type subbursts: range object
        :param samples: A range specifying the samples to be written
        :type samples: range object
        """

        if not self.bursts:
            self.read()

        if not bursts:
            bursts = range(len(self.bursts))

        # We append each burst, so ensure file is empty if it already exists
        with open(path, 'w') as fout:
            pass

        # The ApRES .dat file format is a mixed mode file.  The header is
        # text, and the data section is binary
        for burst in self.bursts[bursts.start:bursts.stop:bursts.step]:
            with open(path, 'a') as fout:
                burst.write_header(fout, subbursts=subbursts, samples=samples)

            with open(path, 'ab') as fout:
                burst.write_data(fout, subbursts=subbursts, samples=samples)

    def burst_to_nc_object(self, burst, nco):
        """
        Map the given burst to the given netCDF object

        The netCDF object can either be a Dataset or a Group

        :param burst: The burst
        :type burst: ApRESBurst
        :param nco: A netCDF object (one of Dataset or Group)
        :type nco: netCDF4._netCDF4.Dataset or netCDF4._netCDF4.Group
        :returns: The netCDF object
        :rtype: netCDF4._netCDF4.Dataset or netCDF4._netCDF4.Group
        """

        # Write the burst header items as netCDF object attributes
        for key in burst.header:
            nco.setncattr(key, burst.header[key])

        for j, key in enumerate(burst.DEFAULTS['data_dim_keys']):
            nco.createDimension(key, burst.data_shape[j])

        data = nco.createVariable(self.DEFAULTS['netcdf_var_name'], burst.data_type, tuple(burst.DEFAULTS['data_dim_keys']))
        data.setncatts(self.DEFAULTS['netcdf_attrs'])
        data[:] = burst.data

        return nco

    def nc_object_to_burst(self, nco):
        """
        Map the given netCDF object to an ApRESBurst object

        The netCDF object can either be a Dataset or a Group

        :param nco: A netCDF object (one of Dataset or Group)
        :type nco: netCDF4._netCDF4.Dataset or netCDF4._netCDF4.Group
        :returns: The burst object
        :rtype: ApRESBurst
        """

        # We make a copy, otherwise data is invalid after file is closed
        data = nco.variables[self.DEFAULTS['netcdf_var_name']][:]
        attrs = vars(nco).copy()

        burst = ApRESBurst()
        burst.data_start = 0              # Stop data being read from file

        # Remove any attributes that weren't part of the original burst's header
        try:
            del attrs['history']
        except KeyError:
            pass

        # Reconstruct the original burst from the parsed header and data.  We
        # initially set the header_lines to be the parsed header which allows us
        # to determine the burst format version, and thus setup the DEFAULTS
        # parsing tokens accordingly.  This then allows us to correctly
        # reconstruct the raw header lines with the appropriate header line
        # delimiter for the burst format version
        burst.data = data
        burst.header = attrs
        burst.header_lines = burst.header
        burst.determine_file_format_version()
        burst.reconstruct_header_lines()
        burst.configure_from_header()

        return burst

    def to_netcdf(self, path=None, mode='w'):
        """
        Write the bursts to the given file path as a netCDF4 file

        The default netCDF file path is the same as the input file, but with
        a .nc suffix

        :param path: The path of the output file
        :type path: str
        :param mode: Mode in which to open the file
        :type mode: str
        """

        if not self.bursts:
            self.read()

        path = path or os.path.splitext(self.path)[0] + self.DEFAULTS['netcdf_suffix']

        ncfile = Dataset(path, mode)

        # Add the command line invocation to global history attribute
        if self.DEFAULTS['netcdf_add_history']:
            ncfile.history = ' '.join(sys.argv)

        # Write each burst as a netCDF4/HDF5 group
        for i, burst in enumerate(self.bursts):
            group = ncfile.createGroup(self.DEFAULTS['netcdf_group_name'].format(i))
            self.burst_to_nc_object(burst, group)

        ncfile.close()




import argparse

from apres import ApRESFile

def main():
    parser = argparse.ArgumentParser(description='print the header, a sample of the data, and diagnostics, from the given raw ApRES file')
    parser.add_argument('infile', help='ApRES raw file')
    args = parser.parse_args()

    with ApRESFile(args.infile) as f:
        for burst in f.read():
            header = burst.header
            print(f'header: header_start = {burst.header_start}')
            print(f'header: data_start = {burst.data_start}')
            print(f'header: data_shape = {burst.data_shape}')
            print(f'header: dict = ')
            print(header)

            data = burst.data
            print(f'data: len(data) = {len(data)}')
            print(f'data: data.size = {data.size}')
            print(f'data: data.shape = {data.shape}')

            # Show a small selection of the first subbursts and samples
            if len(data) > 0:
                nsubbursts = 10 if data.shape[0] > 10 else data.shape[0]
                nsamples = 10 if data.shape[1] > 10 else data.shape[1]
                print('data: head = ')
                print(data[0:nsubbursts, 0:nsamples])

        print(f'number of bursts = {len(f.bursts)}')

if __name__ == '__main__':
    main()




import argparse

from apres import ApRESFile

def main():
    parser = argparse.ArgumentParser(description='convert a raw ApRES file to a netCDF4 file')
    parser.add_argument('infile', help='ApRES raw file')
    parser.add_argument('outfile', help='converted netCDF file', default=None, nargs='?')
    args = parser.parse_args()
 
    with ApRESFile(args.infile) as f:
        f.to_netcdf(args.outfile)

if __name__ == '__main__':
    main()




import os
import argparse

from matplotlib import pyplot as plt
import numpy as np
from netCDF4 import Dataset

from apres import ApRESFile

def construct_radargram(plt, mat, xaxis, yaxis, xlim=None, ylim=None, xlabel='NSubBursts', ylabel='N_ADC_SAMPLES', contrast=1.0, cmap='gray', aspect='auto'):
    """
    Plot the given data as a radargram

    :param plt: The plot object
    :type plt: matplotlib.pyplot
    :param mat: The data with each sub burst (trace) as a column of a matrix
    :type mat: numpy.matrix
    :param xaxis: The x-axis values (of length NSubBursts)
    :type xaxis: numpy.ndarray
    :param yaxis: The y-axis values (of length N_ADC_SAMPLES)
    :type yaxis: numpy.ndarray
    :param xlim: The x-axis limits (defaults to [min(xaxis), max(xaxis)])
    :type xlim: list
    :param ylim: The y-axis limits (defaults to [max(yaxis), min(yaxis)])
    :type ylim: list
    :param xlabel: The x-axis label
    :type xlabel: str
    :param ylabel: The y-axis label
    :type ylabel: str
    :param contrast: The plot contrast factor
    :type contrast: float
    :param cmap: The plot colour map name (can be any supported by matplotlib)
    :type cmap: str
    :param aspect: The plot aspect ratio
    :type aspect: str
    """

    dx = xaxis[1] - xaxis[0]
    dy = yaxis[1] - yaxis[0]
    xlim = xlim or [min(xaxis), max(xaxis)]
    ylim = ylim or [max(yaxis), min(yaxis)]
    extent = [min(xaxis) - dx/2.0, max(xaxis) + dx/2.0, max(yaxis) + dy/2.0, min(yaxis) - dy/2.0]
    std_contrast = np.nanmax(np.abs(mat)[:])

    plt.imshow(mat, cmap=cmap, extent=extent, aspect=aspect, vmin=-std_contrast/contrast, vmax=std_contrast/contrast)

    plt.gca().set_xlabel(xlabel)
    plt.gca().set_ylabel(ylabel)
    plt.xlim(xlim)
    plt.ylim(ylim)

def parse_cmdln():
    """
    Parse the command line options and arguments

    :returns: The parsed command line arguments object
    :rtype: argparse.Namespace object
    """

    epilog = """Examples

Plot the given ApRES .dat file as a radargram:

python3 -m apres.plot_apres filename.dat

Plot the given converted netCDF file as a radargram:

python3 -m apres.plot_apres filename.nc

Plot the given ApRES file as a radargram, increasing the contrast:

python3 -m apres.plot_apres -c 10 filename.dat

Same as above, but with a blue-white-red colour map:

python3 -m apres.plot_apres -c 10 -m bwr filename.dat

Plot the first trace from the given ApRES file:

python3 -m apres.plot_apres -t filename.dat

Plot the first 6 traces, in a 3x2 grid:

python3 -m apres.plot_apres -t -g 3 2 filename.dat
"""

    parser = argparse.ArgumentParser(description='plot ApRES data, either from a .dat file, or a converted netCDF file', epilog=epilog, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('filename', help='filename')

    group = parser.add_mutually_exclusive_group()
    group.add_argument('-r', '--radargram', help='plot a radargram', action='store_const', dest='type', const='radargram')
    group.add_argument('-t', '--traces', help='plot individual traces', action='store_const', dest='type', const='traces')
    parser.set_defaults(type='radargram')

    parser.add_argument('-g', '--grid', help='plot the first nrows x ncols traces', dest='grid', nargs=2, default=[1, 1], type=int)
    parser.add_argument('-c', '--contrast', help='contrast for the radargram', dest='contrast', default=1.0, type=float)
    parser.add_argument('-m', '--cmap', help='colour map for the radargram', dest='cmap', default='gray')

    args = parser.parse_args()

    return args

def plot_data(args, data, title):
    """
    Plot the given data

    :param args: The parsed command line arguments object
    :type args: argparse.Namespace object
    :param data: The data to plot
    :type data: numpy.array
    :param title: The title for the plot
    :type title: str
    """

    if args.type == 'traces':
        (nrows, ncols) = args.grid
        fig = plt.figure()

        for i in range(nrows * ncols):
            ax = fig.add_subplot(nrows, ncols, i + 1)
            ax.plot(data[i])

        fig.supxlabel('Samples')
        fig.supylabel('Amplitude')
    else:
        ntraces = int(data.shape[0])
        npoints = int(data.shape[1])
        xaxis = np.linspace(1.0, float(ntraces), int(ntraces))
        yaxis = np.linspace(0.0, float(npoints), int(npoints))
        cols = np.zeros((npoints, ntraces))

        for i, trace in enumerate(data):
            cols[:,i] = trace

        mat = np.asmatrix(cols)

        # Plot the vertical traces as a radargram
        construct_radargram(plt, mat, xaxis, yaxis, contrast=args.contrast, cmap=args.cmap)

    plt.suptitle(title)
    plt.show()

def plot_burst(args, burst):
    """
    Plot the given burst

    :param args: The parsed command line arguments object
    :type args: argparse.Namespace object
    :param burst: The burst
    :type burst: ApRESBurst
    """

    # Can't plot a radargram for aggregated data
    if int(burst.header['Average']) > 0:
        args.type = 'traces'

    title = f'{os.path.basename(args.filename)}: {burst.header["Time stamp"]}'
    plot_data(args, burst.data, title)

def main():
    args = parse_cmdln()
    suffix = os.path.splitext(args.filename)[1]

    if suffix.lower() == ApRESFile.DEFAULTS['apres_suffix']:
        with ApRESFile(args.filename) as f:
            for burst in f.read():
                plot_burst(args, burst)
    elif suffix.lower() == ApRESFile.DEFAULTS['netcdf_suffix']:
        with Dataset(args.filename, 'r') as f:
            af = ApRESFile()

            for key in f.groups:
                burst = af.nc_object_to_burst(f.groups[key])
                plot_burst(args, burst)
    else:
        raise ValueError(f'Unknown or unsupported file type: {suffix}')

if __name__ == '__main__':
    main()




import argparse

from apres import ApRESFile

def main():
    parser = argparse.ArgumentParser(description='read a raw ApRES file and write to another raw ApRES file, optionally subsetting the data ')
    parser.add_argument('infile', help='ApRES raw file')
    parser.add_argument('outfile', help='ApRES raw file')
    parser.add_argument('-b', '--bursts', help='only write out the first BURSTS bursts', dest='bursts', default=None, type=int)
    parser.add_argument('-u', '--subbursts', help='only write out the first SUBBURSTS subbursts', dest='subbursts', default=None, type=int)
    parser.add_argument('-s', '--samples', help='only write out the first SAMPLES ADC samples', dest='samples', default=None, type=int)
    args = parser.parse_args()

    # We can optionally select a subset of bursts, subbursts and/or samples
    # for output.  These have to be range objects
    if args.bursts:
        args.bursts = range(args.bursts)
    if args.subbursts:
        args.subbursts = range(args.subbursts)
    if args.samples:
        args.samples = range(args.samples)
 
    with ApRESFile(args.infile) as f:
        f.to_apres_dat(args.outfile, bursts=args.bursts, subbursts=args.subbursts, samples=args.samples)

if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['apres']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.4.3,<4.0.0', 'netCDF4>=1.5.2,<2.0.0', 'numpy>=1.17.2,<2.0.0']

entry_points = \
{'console_scripts': ['apres_to_nc = apres.apres_to_nc:main',
                     'nc_to_apres = apres.nc_to_apres:main',
                     'plot_apres = apres.plot_apres:main',
                     'read_apres = apres.read_apres:main',
                     'write_apres = apres.write_apres:main']}

setup_kwargs = {
    'name': 'bas-apres',
    'version': '0.1.0',
    'description': 'Package for working with BAS ApRES files',
    'long_description': "# apres\n\nThis project enables ApRES .dat files to be read, rewritten, and converted.\n\n## The apres package\n\nThe apres package contains a number of classes for working with ApRES .dat files.\n\n* The `ApRESBurst` class manages reading/writing a single burst block (the textual header and the binary data) from/to an opened ApRES .dat file.\n* The `ApRESFile` class is a context manager for ApRES .dat files.  The class wraps the `ApRESBurst` class and can read a variable number of burst blocks from a file (i.e. a single burst file or a timeseries file).  It also contains methods for subsetting an ApRES file, and for converting the file to netCDF4.\n\n## Install\n\nThe package can be installed from PyPI (note the package distribution name):\n\n```bash\n$ pip install bas-apres\n```\n\n## Simple utility scripts\n\nThe package contains a number of scripts for converting from/to netCDF, plotting the data etc.  When installing the package (e.g., via `pip install`) then these scripts are available as commands and added to the `PATH`.  So for instance, you can just call `apres_to_nc`.  If running the scripts from a `clone` of the source code repository, then they need to be run as modules, e.g. `python -m apres.apres_to_nc`.\n\n### apres_to_nc.py\n\nThis script converts an ApRES .dat file to netCDF4.  The default output netCDF filename has the same name as the input .dat file, but with a .nc file suffix.  Optionally an alternative output netCDF filename can be explicitly given.\n\n```bash\npython3 -m apres.apres_to_nc filename.dat [outfile.nc]\n```\n\nThe conversion is a straightforward mapping, in that each burst becomes a netCDF4 group, where the header lines become group attributes, and the data are stored as a `NSubBursts` x `N_ADC_SAMPLES` 2D data array.\n\n### nc_to_apres.py\n\nThis script converts a previously converted netCDF4 file back to the original ApRES .dat file.  The default output ApRES .dat filename has the same name as the input netCDF file, but with a .dat file suffix.  Optionally an alternative output ApRES filename can be explicitly given.\n\n```bash\npython3 -m apres.nc_to_apres infile.nc [outfile.dat]\n```\n\nThe conversion is a straightforward reversal of the original conversion.  For the newer ApRES .dat file format version, this should be identical to the original file.  For the older ApRES .dat file format version, there will likely be small differences in the whitespace in the textual header.  Ignoring these insignificant changes in whitespace (e.g. `diff -awB orig.dat reconstructed.dat`), the files will be identical.\n\n### plot_apres.py\n\nThis script will plot the `N_ADC_SAMPLES` vs `NSubBursts` as a radargram, for each burst in the file (or group for converted netCDF files).  If the `Average` header item > 0 (and so each subburst has been aggregated), then the script will instead plot the first subburst as a single trace.  The file can be either an ApRES .dat file, or a converted netCDF file.\n\n```bash\npython3 -m apres.plot_apres [-h] [-r | -t] [-g GRID GRID] [-c CONTRAST] [-m CMAP]\n                            filename.{dat,nc}\n\nplot ApRES data, either from a .dat file, or a converted netCDF file\n\npositional arguments:\n  filename              filename\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -r, --radargram       plot a radargram\n  -t, --traces          plot individual traces\n  -g GRID GRID, --grid GRID GRID\n                        plot the first nrows x ncols traces\n  -c CONTRAST, --contrast CONTRAST\n                        contrast for the radargram\n  -m CMAP, --cmap CMAP  colour map for the radargram\n\nExamples\n\nPlot the given ApRES .dat file as a radargram:\n\npython3 -m apres.plot_apres filename.dat\n\nPlot the given converted netCDF file as a radargram:\n\npython3 -m apres.plot_apres filename.nc\n\nPlot the given ApRES file as a radargram, increasing the contrast:\n\npython3 -m apres.plot_apres -c 10 filename.dat\n\nSame as above, but with a blue-white-red colour map:\n\npython3 -m apres.plot_apres -c 10 -m bwr filename.dat\n\nPlot the first trace from the given ApRES file:\n\npython3 -m apres.plot_apres -t filename.dat\n\nPlot the first 6 traces, in a 3x2 grid:\n\npython3 -m apres.plot_apres -t -g 3 2 filename.dat\n```\n\n### read_apres.py\n\nThis script will read the given ApRES .dat file, and for each burst, print the results of parsing the header, such as the dimensions of the data array, and the parsed header dictionary.  It will also *head* the data section (by default the first 10 samples of the first 10 subbursts), to give an initial simple look at the data.\n\nThe script's primary purpose is as a simple example of how to use the `ApRESFile` class to read an ApRES .dat file.\n\n```bash\npython3 -m apres.read_apres filename.dat\n```\n\n### write_apres.py\n\nThis script will read the given input ApRES .dat file, and for each burst, write the header and data to the given output ApRES .dat file.  Optionally a subset of bursts can be written out, specified as the first `bursts` bursts of the input file.  In addition a subset of each burst can be written out, specified as the first `subbursts` subbursts, and the first `samples` ADC samples of these subbursts.  If `bursts`, `subbursts` and `samples` are not specified, then the output file is identical to the input file.\n\nThe script's primary purpose is as a simple example of how to use the `ApRESFile` class to rewrite an ApRES .dat file.\n\n```bash\npython3 -m apres.write_apres [-b BURSTS] [-u SUBBURSTS] [-s SAMPLES] infile.dat outfile.dat\n```\n\n",
    'author': 'Paul Breen',
    'author_email': 'pbree@bas.ac.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/antarctica/bas-apres',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
